@@ -22,9 +22,9 @@ use Net::LDAP::Constant qw(LDAP_SUCCESS
 			   LDAP_INAPPROPRIATE_AUTH
 			);
 
-$VERSION 	= 0.251;
+$VERSION 	= "0.27";
 @ISA     	= qw(Net::LDAP::Extra);
-$LDAP_VERSION 	= 2;      # default LDAP protocol version
+$LDAP_VERSION 	= 3;      # default LDAP protocol version
 
 # Net::LDAP::Extra will only exist is someone use's the module. But we need
 # to ensure the package stash exists or perl will complain that we inherit
@@ -96,7 +96,14 @@ sub new {
   my $arg  = &_options;
   my $obj  = bless {}, $type;
 
-  $obj->_connect($host, $arg) or return;
+  foreach my $h (ref($host) ? @$host : ($host)) {
+    if ($obj->_connect($h, $arg)) {
+      $obj->{net_ldap_host} = $h;
+      last;
+    }
+  }
+
+  return undef unless $obj->{net_ldap_socket};
 
   $obj->{net_ldap_host}    = $host;
   $obj->{net_ldap_resp}    = {};
@@ -117,10 +124,11 @@ sub _connect {
   my ($ldap, $host, $arg) = @_;
 
   $ldap->{net_ldap_socket} = IO::Socket::INET->new(
-    PeerAddr => $host,
-    PeerPort => $arg->{port} || '389',
-    Proto    => 'tcp',
-    Timeout  => defined $arg->{timeout}
+    PeerAddr   => $host,
+    PeerPort   => $arg->{port} || '389',
+    Proto      => 'tcp',
+    MultiHomed => $arg->{multihomed},
+    Timeout    => defined $arg->{timeout}
 		 ? $arg->{timeout}
 		 : 120
   );
@@ -253,7 +261,6 @@ sub bind {
 
     # Save data, we will need it later
     $mesg->_sasl_info($stash{name},$control,$sasl_conn);
-    $ldap->{sasl} = $sasl_conn;
   }
 
   $stash{authentication} = { $auth_type => $passwd };
@@ -578,7 +585,7 @@ sub extension {
     if $ldap->{net_ldap_version} < 3;
 
   $mesg->encode(
-    extendedRequest => {
+    extendedReq => {
       requestName  => $arg->{name},
       requestValue => $arg->{value}
     },
@@ -802,7 +809,7 @@ sub start_tls {
   require Net::LDAPS;
   $arg->{sslversion} = 'tlsv1' unless defined $arg->{sslversion};
   IO::Socket::SSL::context_init( { Net::LDAPS::SSL_context_init_args($arg) } );
-  (IO::Socket::SSL::socketToSSL($sock) and tie *{$sock}, 'IO::Socket::SSL', $sock)
+  IO::Socket::SSL::socketToSSL($sock, {Net::LDAPS::SSL_context_init_args($arg)})
     ? $mesg
     : _error($ldap, $mesg, LDAP_OPERATIONS_ERROR, $@);
 }
